{- Primitive parser combinators
 -
 - Author(s): Doaitse Swierstra
 - Version 1.0, May 14, 1998: first appearance on the software web site.
 -}

{- BEGIN_GHC_ONLY
-- seq on function values only works with ghc-4.00
#define seq (\ _ y -> y)
    END_GHC_ONLY -}

module UU_Parsing where

import List

infixl 2 <?> ; infixl 3 <|> ; infixl 4 <*>

class (Eq s, Show s) => Symbol s

-- primitive parsers

succeed  :: Symbol s => a -> Parser s a
symbol   :: Symbol s => s -> Parser s s


-- primitive parser combinators

(<?>)  :: Symbol s => Parser s a -> (a, String) -> Parser s a
(<|>)  :: Symbol s => Parser s a -> Parser s a -> Parser s a
(<*>)  :: Symbol s => Parser s (b->a) -> Parser s b      -> Parser s a


{- YOU PROBABLY DON'T WANT TO LOOK AT THE CODE BELOW!!
 - look at the article and the complete version of the
 - code instead 
 -}
   
type Empty s a = Bool
type First s a = [s]
type Input s   = [s]

--combine :: Symbol s => Empty s a -> [s] -> [s] -> [s] 
combine e s1 s2 = s1 `union` (if e then s2 else [])

locate i [] notfound = notfound
locate i (Look s p : rest) notfound
      | i == s = p
      | otherwise = locate i rest notfound

--
-- ERROR CORRECTING PARSERS
--

type Parser s a = (EmptyDescr s a, First s a, ParserTab s a) 
                       
data EmptyDescr s a = IsEmpty a
                    | Insert a String

data Look      s a  = Look s a
type ParserTab s a  = [Look s (ErrParFun s a)]

mapParser tab f     = [Look s (f p) | Look s p <- tab]

type ErrParFun s a  = State s -> Noskip s -> (a, State s)
type State s  = ([s],String)
type Noskip s = [[s]]

edempty  v = IsEmpty v
edsymbol s = Insert s (inserted s)
inserted s = show s ++ " inserted" 

ederr (v,msg) = Insert v (inserted msg)

ed1 `edalt` ed2 =
  case ed1 of
    IsEmpty _  -> ed1
    Insert _ _ -> ed2
ed1 `edseq` ed2 =
  case ed1 of
    IsEmpty v1
      -> case ed2 of
           IsEmpty v2    -> IsEmpty (let r = v1 v2 in seq r r)
           Insert  v2 s2 -> Insert  (let r = v1 v2 in seq r r) s2
    Insert v1 s1 ->
      Insert (case ed2 of
                IsEmpty v2   -> let r = v1 v2 in seq r r
                Insert  v2 _ -> let r = v1 v2 in seq r r
             )
             (case ed2 of
                IsEmpty _    -> s1
                Insert  _ s2 -> s1++s2
             )

-- This is  the "most lazy formulation"!
succeed  v = (edempty  v, [], [])

symbol s = (edsymbol s, [ s ], tsymbol s)
tsymbol   s = [Look s (\((t:inp), str) _ -> (t, (inp, str)))]

ep <?> x  = ep <|> (ederr x, [], [])

(ed1, f1, t1) <|> (ed2, f2, t2)
  = (ed1 `edalt` ed2, f1 `union` f2, t1 ++ t2)

choose' (edp,_) state@([],_) noskip = insorempty edp state "endoffile"
choose' x@(edp, ptree) state@((s:inp), errors) noskip
  = locate s ptree
     (\_ _
        -> if any (s `elem`) noskip then insorempty edp state (show s)
           else choose' x (inp, errors ++ deleted s) noskip
     ) state noskip
-- choose' _ _ _ = error "no good alternative"

deleted  s = show s ++ " deleted\n"
insorempty edp state@(input,errors) next
  = case edp of
      IsEmpty pv      -> (pv, state)
      Insert pv error -> (pv, (input
                              , errors ++ error++ " before " ++ next ++"\n"))

(ed1, f1, t1) <*> ~(ed2, f2, t2)
  = (ed1 `edseq` ed2, f1 `fseq` f2, t1 `tseq` t2)
    where 
 f1 `fseq` f2 = f1 `union` case ed1 of IsEmpty _  -> f2 
                                       Insert _ _ -> []
 t1 `tseq` t2
      = mapParser t1
        (\p ss noskip ->
          let  (v1,rs)  = p ss (f2:noskip)
               (v2,rrs) = choose' (ed2, t2) rs noskip
          in (let r = v1 v2 in seq r r, rrs)
        )
       ++ case ed1 of IsEmpty v1 -> mapParser t2
                                   (\q ss noskip ->
                                     let (v2, rs) = q ss noskip
                                     in  (let r = v1 v2 in seq r r , rs)
                                   )
                      Insert _ _ -> []

invokeErr (edp, _, look_table) input noskip =
  let (a, (rest, errors)) = 
         choose' (edp, look_table) (input, []) noskip
  in (a, rest, errors)
