// PanSlider.cpp: implementation of the CPanSlider class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PanSlider.h"
#include <windowsx.h>
#include <string.h>
#include <tchar.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPanSlider::CPanSlider ( HWND           hWndParent
                       , LPCTSTR        label
                       , float          min
                       , float          max
                       , FloatUpdater*  pUpd
                       , void*          pParam
                       , CInvalidator*  pInvalid
    )
{
    TCHAR valStr[255];

    m_min         = min;
    m_max         = max;
    m_updateValue = pUpd;
    m_pParam      = pParam;
    m_iProc       = pInvalid;
    m_inFlux      = false;

    // Get the initial value from the underlying object,
    // since it may have changed from the initial value
    // (this will happen if it has been activated from a
    //  settings file rather than having been created with
    //  default values.)

    m_value       = m_updateValue->GetFloat(m_pParam);

    _stprintf(valStr, TEXT("%2.2f"), m_value);

    Create(hWndParent, label, valStr, 100,20);
 

    ::Edit_SetText(GetDlgItem(ID_EDIT_FIELD), valStr); 

}

CPanSlider::~CPanSlider()
{

}


//////////////////////////////////////////////////////////////////////
// Event handlers
//////////////////////////////////////////////////////////////////////
LRESULT
CPanSlider::OnHScroll(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    DWORD ev = LOWORD(wParam);
      
    if ( ev == TB_ENDTRACK || ev == TB_THUMBTRACK || ev == TB_THUMBPOSITION) {
        TCHAR msg[80];
        int val;
        float valF;
	
        val = SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
        valF = m_min + (m_max - m_min)*((float)val/(float)SLIDER_RANGE);
        // Only update if the value has changed.  Oddly, this test always
        // succeeds.  ToDo: why?
        if (valF != m_value) {
            _stprintf(msg, TEXT("%2.2f"), valF);

            m_value = valF;

            // update the edit field with new value.
            m_inFlux = true;
            ::Edit_SetText(GetDlgItem(ID_EDIT_FIELD), msg); 
            m_inFlux = false;

            // .. and the view window
            m_updateValue->UpdateFloat(m_pParam,valF);
            m_iProc->Invalidate();
        }
    }

    return 0;
}


LRESULT
CPanSlider::OnChange(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    USES_CONVERSION;

    TCHAR valStr[255];
    int pos, val;
    float valF;
      
    if ( m_inFlux != true ) {
        // the trackbar is not being dragged, so this event was caused
	    // by the user modifying the edit box value directly.
        ::Edit_GetText(hWndCtl, valStr, 255);
        valF = (float)atof(T2A(valStr));
        m_updateValue->UpdateFloat(m_pParam, valF);
	    m_iProc->Invalidate();
    } else {
	    valF = m_value;
    }

    val = SendMessage(hWndCtl, TBM_GETPOS, 0, 0);
 	pos = SLIDER_MIN_RANGE + 
	          (int) ((float)SLIDER_RANGE * ((valF - m_min)/ (m_max - m_min)));

    if ( m_inFlux != true && val != pos ) {
        // If new value was typed in & is different to current,
	    // then we'll need to update the trackbar thumb.
	    pos = (pos > SLIDER_MAX_RANGE ? SLIDER_MAX_RANGE : pos);
	    pos = (pos < SLIDER_MIN_RANGE ? SLIDER_MIN_RANGE : pos);
	    SendMessage(GetDlgItem(ID_TRACKBAR), TBM_SETPOS, TRUE, pos);
    }

    return 0;
}
