module Arrays where

import Color
import Exp

---- Arrays (e.g., for bitmaps)

-- 1D and 2D arrays, with size and subscriptor.

data Array1 c = Array1 IntE (IntE -> c)

data Array2 c = Array2 IntE IntE ((IntE, IntE) -> c)

arr1Map :: (a -> b) -> Array1 a -> Array1 b
arr1Map f (Array1 w g) = Array1 w (f.g)

arr2Size :: Array2 c -> (IntE, IntE)
arr2Size (Array2 w h _) = (w,h)

arr2Width  :: Array2 c -> IntE
arr2Width  (Array2 w _ _) = w

arr2Height :: Array2 c -> IntE
arr2Height (Array2 _ h _) = h

arr2Map :: (a -> b) -> Array2 a -> Array2 b
arr2Map f (Array2 w h g) = Array2 w h (\(i,j) -> f (g (i,j)))

-- Note: no bounds checking.
indexArray2 :: Array2 c -> (IntE,IntE) -> c
indexArray2 (Array2 _ _ f) (x,y) = f (x,y)

array1 :: ExpType a => IntE -> (IntE -> Exp a) -> Array1 (Exp a)
array1 size elem = Array1 size (readArr (mkArr size elem))

array2 :: ExpType a => (IntE,IntE) -> ((IntE,IntE) -> Exp a) -> Array2 (Exp a)
array2 (w,h) f = Array2 w h (\ (i,j) -> readArr (readArr arr j) i)
 where
   arr = mkArr h (\ j -> mkArr w (\ i -> f (i,j)))


-- Turn virtual arrays into real ones.  This transformation can cause work
-- to be done sooner and cached.
reifyArray1 :: ExpType a => Array1 (Exp a) -> Array1 (Exp a)
reifyArray1 (Array1 w sub) = array1 w sub

reifyArray2 :: ExpType a => Array2 (Exp a) -> Array2 (Exp a)
reifyArray2 (Array2 w h sub) = array2 (w,h) sub

reifyArray1Color :: Array1 ColorE -> Array1 ColorE
reifyArray1Color col_arr =
  arr1Map fromBGRA32 (reifyArray1 (arr1Map toBGRA32 col_arr))

reifyArray2Color :: Array2 ColorE -> Array2 ColorE
reifyArray2Color col_arr =
  arr2Map fromBGRA32 (reifyArray2 (arr2Map toBGRA32 col_arr))
