module PP where

import qualified Pretty as P
import qualified PrettyUtils as PU  -- sigh, Hugs98 doesn't allow two modules
				    -- to be mapped to the same 'as' 

import List ( intersperse )
import BasicTypes

infixl 6 <+>
infixl 5 $$

------ Pretty printing

type PPDoc = Context -> P.Doc

type Context
 = ( Bool  -- True => on the right of a binary operator.
   , Assoc -- associativity of operator we're appearing as expr of.
   , Int   -- current precedence
   )

-- Lifting the pretty printing combinators needed here.
text x = const (P.text x)

parens x = \ p -> P.parens (x p)

a <+> b  = \ x -> (P.<+>) (a x) (b x)
a <> b   = \ x -> (P.<>) (a x) (b x)
($$) a b = \ x -> (P.$$) (a x) (b x)
($+$) a b = \ x -> (P.$+$) (a x) (b x)

nest x d = \ p -> P.nest x (d p)

hang d1 x d2 = \ p -> P.hang (d1 p) x (d2 p)
char c = const (P.char c)

hsep xs = \ p -> P.hsep (map (flip ($) p) xs)
hcat xs = \ p -> P.hcat (map (flip ($) p) xs)
vcat xs = \ p -> P.vcat (map (flip ($) p) xs)
sep xs  = \ p -> P.sep  (map (flip ($) p) xs)

comma  = \ _ -> P.comma
equals = \ _ -> P.equals
semi   = \ _ -> P.semi

empty  = \ _ -> P.empty

onRight :: PPDoc -> PPDoc
onRight pd = \ (_,as,prec) -> pd (True,as,prec)

ppOperator :: (Int    -- precedence
              ,Assoc) -- associativity
           -> PPDoc
           -> PPDoc
ppOperator (p2,as2) pd (onR,as1,p1)
  | needParens = parens pd new_context
  | otherwise  = pd new_context
 where
  new_context = (False,as2,p2)
  
  needParens  =
    p1 > p2   ||                -- if a lower-precedence operator is appearing
    		                -- as an arg. to a higher prec. one, we do need parens.
    (p1 == p2 &&                -- same precedence ops will need parens, iff:
     ((isRA_op1 /= isRA_op2)  || --   * they have opposing associativity.
      (onR && not (isRA_op1)) || --   * the new operator appears on the right of a l. assoc operator.
      (not onR && isRA_op1)))    --   * the new op appears on the left of a r. assoc operator.
     
  isRA_op1 = as1 == RAssoc
  isRA_op2 = as2 == RAssoc
	   
{-
withPrec :: Int -> PPDoc -> PPDoc
withPrec p1 pd = \ p2 ->
  (if p2 > p1 then parens pd else pd) p1

-- Override.  See, e.g., ppLoop in DExp.hs
atPrec :: Int -> PPDoc -> PPDoc
atPrec p1 pd = const (pd p1)
-}
-- end PP ops

-- pretty printing colors and expressions

showPPDoc :: PPDoc -> Int -> String
showPPDoc x p = show (x (False,NAssoc,p))

-- The default line width is 100.  We could change to 72 via the
-- following, but Hugs is missing renderStyle and Style
-- 
-- showPPDoc x = P.renderStyle (P.Style 72 1.5 P.PageMode) (x 9)

ppArg x = text ("arg"++show x)

ppVar x = text ("local"++show x)

ppInt x = text (show x)
ppDouble x = text (show x) <> char 'F'
ppBool x = text (show x)

isEmpty :: PPDoc -> Bool
isEmpty p = let x = p (False,NAssoc,0) in P.isEmpty x

vsepWith d ds = \ p -> PU.vsepWith (d p) (map (flip ($) p) ds)
vsep ls = \ p -> PU.vsep (map (flip ($) p) ls)
ppDecls ls = \ p -> PU.ppDecls (map (flip ($) p) ls)
ppTuple ls = \ p -> PU.ppTuple (map (flip ($) p) ls)

ppTupleVert ls = \ p -> PU.ppTupleVert (map (flip ($) p) ls)

stmtBlock n ds = char '{' $+$ nest n ds $$ char '}'

