-- Misc text-based demos
--
-- ToDo: move all these defs elsewhere and eliminate this module

module Text where

import Monad
import Pan
import Shadow (lightBg, lightBgD, shadowUI, addShadowUI, shadowedBmpUI, shadowedBmpUI')
import Simple (swirlUI,tileRotateUI,tileSizeUI,tileUI,tileUI',spinUI)
import MiscDemos (ybRingsUI, circlify, circlify', wibbleUI, woobleUI   )
import Wavies (washerUI)

textRegionUI :: UI Region
textRegionUI = liftM (chromaReg 0.05 white) (textBox "text cutout")
     
swirlTextRegionUI :: UI Region
swirlTextRegionUI = swirlUI `ap` textRegionUI

swirlTextCutoutUI :: UI FilterC
swirlTextCutoutUI = liftM crop swirlTextRegionUI

swirlTextCutoutTiledUI = swirlTextCutoutUI `ap` tileUI' -- ybRingsUI

-- The defs above don't exploit the text's antialiased rendering.
-- This next set has nice soft edges, but must be used with black text to
-- get opacity in the solid parts of the fonts.

softCrop :: Image FractionE -> ImageC -> ImageC
softCrop = fade

textAlphaUI :: UI (Image FractionE)
textAlphaUI = liftM darknessI' (textBox "text cutout")
 where
   -- Like darknessI, but extends the image with white
   darknessI' = darknessI . lift1 transpToWhite
   transpToWhite c = ifE (isInvisible c) white c
   isInvisible c = a ==* 0 where (_,_,_,a) = colorToBGRA c

swirlTextAlphaUI :: UI (Image FractionE)
swirlTextAlphaUI = swirlUI `ap` textAlphaUI

swirlTextSoftCutoutUI :: UI FilterC
swirlTextSoftCutoutUI = liftM softCrop swirlTextAlphaUI

swirlTextSoftCutoutTiledUI = swirlTextSoftCutoutUI `ap` tileUI' -- ybRingsUI



-- lightBg only works correctly with black text, since it maps black to
-- opaque.  whiteBg leaves an antialiasing ring around the characters'
-- peripheries.

{-
shadowedTextUI =
  do (w,h,im)   <- textBoxSize "Shadows"
     withShadow <- addShadowUI black (-h/2) (lightBg {-whiteBg 0.05-} im)
     return $ withShadow `over` lerpI (const) ybRings whiteI
-}

shadowedTextUI =
  do (w,h,im)   <- textBoxSize "Shadows"
     withShadow <- addShadowUI black (-h/2) (lightBg {-whiteBg 0.05-} im)
     -- fadeBack   <- slider "fade background" (0,1) 0.75
     return $ withShadow -- `over` lerpI (const fadeBack) ybRings whiteI

-- I don't know why, but shadowedTextUI doesn't work with ybRings as
-- background.  There are visual anomalies in the transparent part of the
-- shadow image over the most yellow and blue parts of the background.

{-

How to do soft shadows?  I think we want to make a soft monochrome
version of the foreground image and then shear it.  That way, we know
what size to use for the convolution.

Bilerp gives a way to soften.  How much should we use it to augment
convolution?  (Could we use it instead?)

We should really exploit the anti-aliasing done in text rendering.  What
stops us?  Mainly not knowing what color the text is and hence how to
convert to monochrome with opacity.  If we assume black, then we can use
the lightBg function to preserve anti-aliasing.  Later, we can replace the
color or do an "alpha crop" with some other image.

-}

{-
softShadowedTextUI =
  do (w,h,im) <- textBoxSize "Shadows"
     shadow   <- shadowUI black (-h/2) (whiteBg 0.05 im)
     softness <- slider "shadow softness" (1,20) 5
     let ker      = uscale softness $ (gauss (1/2) . distOsq)
         softened = convolve softness (w,h) ker shadow
     return $ shadow `over` lerpI (const 0.75) ybRings whiteI
-}

softDiskUI1 =
  do softness <- islider "softness" (1,10) 5
     let ker = uscale (i2f softness) (gauss (1/2) . distOsq)
         im = diskIm 50 red
     return $ convolveCC softness (100,100) ker im

softDiskUI2 =
  do softness <- islider "softness" (1,10) 5
     let ker = uscale (i2f softness) (gauss (1/2) . distOsq)
         bmp = pixelize (100,100) (diskIm 50 red)
     return $ convolveDC softness ker bmp

softDiskUI3 =
  do softness <- islider "softness" (1,10) 5
     let ker = uscale (i2f softness) (gauss (1/2) . distOsq)
         bmp = reifyArray2Color $ pixelize (100,100) (diskIm 50 red)
     return $ convolveDC softness ker bmp

-- The previous soft disks work fine, except that even the interior of the
-- disk lightens as the softness increases.

softDiskUI4 =
  do softness <- islider "softness" (1,10) 5
     r        <- islider "disk radius" (20,100) 50
     let s   = 2*r
         ker = uscale (i2f softness) (gauss (1/2) . distOsq)
         bmp = reifyArray2Color $ pixelize (s,s) (diskIm (i2f r) red)
     return $ convolveDC softness ker bmp


softTextUI =
  do bmp      <- textBoxD "Soft Text"
     softness <- slider "softness" (0,10) 3
     let ker = uscale softness (gauss (1/2) . distOsq)
     return $ convolveDC (roundE softness) ker bmp

softShadowedTextUI =
  do bmp <- textBoxD "soft shadows"
     shadowedBmpUI' bmp lightBgD


wibbleTextUI, woobleTextUI :: UI ImageC
wibbleTextUI = wibbleUI `ap` textBox "wibble text"
woobleTextUI = woobleUI  `ap` textBox "wooble text"

circleTextUI :: UI ImageC
circleTextUI  = liftM circlify (textBoxSize "You can write text in circle.  ")

circleTextTurningUI :: UI ImageBC
circleTextTurningUI  = spinUI `ap` circleTextUI

washerTextUI :: UI ImageBC
washerTextUI  = washerUI  `ap` textBox "washer text"


-- circleTextTurningUI = liftM2 f circleTextUI (slider "rate" (-5,5) 0)
--  where
--    f im r = \ t -> rotate (-r*t) im

-- Rotation would be better if we had integration.  Otherwise, spins when
-- making small rate adjustments.  ToDo: make an integrator pseudo UI
-- element.  Stateful integration would happen in the viewer.

-- Tiled version
-- circleTextTurningTiledUI1 = tileUI `comp` circleTextTurningUI

circleTiledUI' :: SizedImageC -> ImageC
circleTiledUI' = tileSizedIm . circlify'

circleTiled'' :: SizedImage a -> FloatE -> Image a
circleTiled'' sim t = tileSizedIm $ rotateSIm t $ circlify' sim
 where rotateSIm ang (w,h,im) = (w,h, rotate ang im) 

circleTextTiledUI' =
  liftM (tileSizedIm . circlify') (textBoxSize "round ")

circleTextTurningTiledUI =
 do sim <- textBoxSize "round "
    ang <- liftM (\ r t -> -r*t) (slider "rate" (-5,5) 2)
    return $
      \ t -> circleTiled'' sim (ang t)


tileSizedIm (w,h,im) = tile (w,h) im

--tileCirclifyUI = tileUI'' (liftM circlify' imageToTileUI)
tileCirclifyTextUI     = tileSizeUI (liftM circlify' (textBoxSize "text goes here"))


{-

Maybe change convolve to take real-valued radius and size.  Also, explore
a balance of convolution and bilerp.

For examples like this one, it's probably better to work more at the
discrete level.  Use textBoxD and convolveD

-}

makeAll = makeAllFor allBackends

makeAllFor bends = do
  compileImageFor  bends "shadowed text"           shadowedTextUI
  compileImageFor  bends "soft text"               softTextUI
  compileImageFor  bends "soft shadowed text"      softShadowedTextUI
  compileImageFor  bends "swirl text cutout tiled" swirlTextCutoutTiledUI
  compileImageFor  bends "swirl text soft cutout tiled" swirlTextSoftCutoutTiledUI
  compileImageFor  bends "wibble text"             wibbleTextUI
  compileImageFor  bends "wooble text"             woobleTextUI
  compileImageBFor bends "washer text"             washerTextUI
  compileImageFor  bends "circle text"             circleTextUI
  compileImageBFor bends "circle text turning"     circleTextTurningUI
  compileImageFor  bends "circle text tiled"       tileCirclifyTextUI
  compileImageBFor bends "circle turning text tiled" circleTextTurningTiledUI
  --compileImageBFor bends "circle text turning tiled" circleTextTurningTiledUI
